
function ToonInfo.CreateScrollableRiftWindow(name, title, context)
	local newWindow

	newWindow=UI.CreateFrame("RiftWindow", name, context)
	newWindow:SetPoint("CENTER", UIParent, "CENTER", 0,0)
	newWindow:SetController("content")
	newWindow:SetWidth(440)
	newWindow:SetTitle(title)
	newWindow:SetBackgroundColor(0, 0, 0, 1)

	local closeButton = UI.CreateFrame("RiftButton", name.."CloseButton", newWindow)
	closeButton:SetSkin("close")
	closeButton:SetPoint("TOPRIGHT", newWindow, "TOPRIGHT", 0, -40)
	function closeButton:EventLeftClick(handle)
		newWindow:SetVisible(false)
	end
	closeButton:EventAttach(Event.UI.Input.Mouse.Left.Click, closeButton.EventLeftClick, "LeftClick", -2)

	local mask=UI.CreateFrame("Mask", name.."Mask", newWindow)
	mask:SetPoint("TOPLEFT", newWindow, "TOPLEFT", 0, 0)
	mask:SetPoint("BOTTOMRIGHT", newWindow, "BOTTOMRIGHT", -15, 0)
	
	local content=UI.CreateFrame("Frame", name.."Content", mask)
	content:SetAllPoints()

	local scrollbar = UI.CreateFrame("RiftScrollbar", name.."Scrollbar", newWindow)
	scrollbar:SetOrientation("vertical")
	scrollbar:SetLayer(10)

	scrollbar:SetPoint("TOPRIGHT", newWindow, "TOPRIGHT", 0, 0)
	scrollbar:SetPoint("BOTTOMRIGHT", newWindow, "BOTTOMRIGHT", 0, 0)
	scrollbar:SetWidth(15)

	function scrollbar:EventScrollbarChange(handle)
		local pos=scrollbar:GetPosition()
		-- print("positioning to "..pos)
		content:SetPoint("TOPLEFT", newWindow, "TOPLEFT", 0, -pos)
	end
	scrollbar:EventAttach(Event.UI.Scrollbar.Change, scrollbar.EventScrollbarChange, "ScrollChange", -2)
	
	function content:EventScrollUp(handle)
		if not scrollbar:GetVisible() then return end
		local pos=scrollbar:GetPosition()
		pos = pos-30;
		if (pos<0) then pos=0 end
		scrollbar:SetPosition(pos)
	end
	content:EventAttach(Event.UI.Input.Mouse.Wheel.Forward, content.EventScrollUp, "ScrollUp", -2)

	function content:EventScrollDown(handle)
		if not scrollbar:GetVisible() then return end
		local pos=scrollbar:GetPosition()
		local min, max=scrollbar:GetRange()
		if (max<0) then return end
		pos=pos+30;
		if (pos>max) then pos=max end
		scrollbar:SetPosition(pos)
	end
	content:EventAttach(Event.UI.Input.Mouse.Wheel.Back, content.EventScrollDown, "ScrollDown", -2)

	local border = newWindow:GetBorder()
	function border:EventLeftDown(handle)
		self.leftDown = true
		local mouse = Inspect.Mouse()
		self.originalXDiff = mouse.x - self:GetLeft() - 18
		self.originalYDiff = mouse.y - self:GetTop() - 60
		local left, top, right, bottom = newWindow:GetBounds()
		newWindow:ClearAll()
		newWindow:SetPoint("TOPLEFT", UIParent, "TOPLEFT", left, top)
		newWindow:SetWidth(right-left)
		newWindow:SetHeight(bottom-top)		
	end
	border:EventAttach(Event.UI.Input.Mouse.Left.Down, border.EventLeftDown, "LeftDown", -2)

	function border:EventLeftUp(handle)
		self.leftDown = false
	end
	border:EventAttach(Event.UI.Input.Mouse.Left.Up, border.EventLeftUp, "LeftUp", -2)
	border:EventAttach(Event.UI.Input.Mouse.Left.Upoutside, border.EventLeftUp, "LeftUp", -2)

	function border:EventMouseMove(handle, x, y)
		if not self.leftDown then
			return
		end
		newWindow:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x - self.originalXDiff, y - self.originalYDiff)
	end	
	border:EventAttach(Event.UI.Input.Mouse.Cursor.Move, border.EventMouseMove, "MouseMove", -2)
		
	function content:SetVisible(flag)
		newWindow:SetVisible(flag)
	end
	
	function content:GetVisible()
		return newWindow:GetVisible()
	end
	
	function content:SetWidth(x)
		newWindow:SetWidth(x+15)
	end
	
	function content:SetHeight(x)
		local frameHeight=newWindow:GetHeight()
		if (x <= frameHeight) then
			scrollbar:SetVisible(false)
		else
			-- print("need height ".. x .. " frame has " .. frameHeight .. ": set scrollbar to " .. (x-frameHeight))
			scrollbar:SetRange(0, x-frameHeight)
			scrollbar:SetVisible(true)
		end
	end

	return content
end
