local ToonInfoAddonData, PrivateTable = ...

if ToonInfo then
	print ("ToonInfo already loaded")
	return
end

ToonInfo = {
	version=ToonInfoAddonData.toc.Version
}

local currentToon

local function L(x) return Translations.ToonInfo.L(x) end

local function ensureVariablesInited()
	if not ToonInfoChar then ToonInfoChar={} end	
	if not ToonInfoShard then ToonInfoShard={} end
	if not ToonInfoGlobal then ToonInfoGlobal={} end
	if not ToonInfoGlobal["currencyicon"] then ToonInfoGlobal["currencyicon"]={} end
	if not ToonInfoGlobal["currencyname"] then ToonInfoGlobal["currencyname"]={} end
	if not ToonInfoGlobal["currencycategory"] then ToonInfoGlobal["currencycategory"]={} end
	if not ToonInfoGlobal["factionname"] then ToonInfoGlobal["factionname"]={} end
	if not ToonInfoGlobal["factioncategory"] then ToonInfoGlobal["factioncategory"]={} end
end

local function itemChanged(handle, updates)
	if not currentToon then		-- safety measure
		return
	end
	ensureVariablesInited()
	if not ToonInfoShard[currentToon.name] then
		ToonInfoShard[currentToon.name] = {}
	end
	if not ToonInfoShard[currentToon.name]["slots"] then
		ToonInfoShard[currentToon.name]["slots"] = {}
	end
	ToonInfoShard[currentToon.name]["guild"]=false
	if currentToon.guild then
		if not ToonInfoShard[currentToon.guild] then
			ToonInfoShard[currentToon.guild] = {}
		end
		if not ToonInfoShard[currentToon.guild]["slots"] then
			ToonInfoShard[currentToon.guild]["slots"] = {}
		end
		ToonInfoShard[currentToon.guild]["guild"]=true
	end
	local quieted=false
	for k, v in pairs(updates) do
		--if k:sub(1, 2) == "sv" or k:sub(1, 4) == "sbmn" then
		--	print (k .. ": " .. (v and v or "false"));
		--end
		if k:sub(1, 2) == "sg" then
			index=currentToon.guild
			if not index then
				print ("strange .. guild bank but no guild?")
				return
			end
			if not quieted then
				quieted=true
				Command.System.Watchdog.Quiet()
			end			
		else
			index=currentToon.name
		end
		if (v == "nil") then		-- guild bank closed
			if ToonInfoShard[index]["slots"][k] then
				ToonInfoShard[index]["slots"][k].OutOfDate=true
			end
		elseif (v) then
			ToonInfoShard[index]["slots"][k]=Inspect.Item.Detail(v)
		else
			ToonInfoShard[index]["slots"][k]=v
		end
	end
end

local function currencyChanged(handle, currencies)
	if not currentToon then		-- safety measure
		return
	end
	ensureVariablesInited()
	if not ToonInfoShard[currentToon.name] then
		ToonInfoShard[currentToon.name] = {}
	end
	if not ToonInfoShard[currentToon.name]["money"] then
		ToonInfoShard[currentToon.name]["money"] = {}
	end
	for k, v in pairs(currencies) do
		ToonInfoShard[currentToon.name]["money"][k]=v
		ToonInfo.UpdateMoneyWindow(currentToon.name, k, v)
	end
end

local function factionChanged(handle, factions)
	if not currentToon then		-- safety measure
		return
	end
	ensureVariablesInited()
	if not ToonInfoShard[currentToon.name] then
		ToonInfoShard[currentToon.name] = {}
	end
	if not ToonInfoShard[currentToon.name]["faction"] then
		ToonInfoShard[currentToon.name]["faction"] = {}
	end
	for k, v in pairs(factions) do
		ToonInfoShard[currentToon.name]["faction"][k]=v
		ToonInfo.UpdateFactionWindow(currentToon.name, k, v)
	end
end

-- This should go to VariablesLoaded, or to AddonLoaded, but for the
-- first few frames, Inspect.Unit.Detail("player") returns nil.
-- So we use the update cycle to retry till we get something useful.

local function systemUpdate(handle)
	local items, currencies, factions
	if not currentToon then
		-- If the player is in fight directly after logging in, delay this, because we can't quiet the watchdog.
		if Inspect.System.Secure() then return end
		currentToon=Inspect.Unit.Detail("player")
		if not currentToon or currentToon.availability ~= "full" then
			currentToon=nil
			return
		end
		-- Normally not a good idea to call this in a frame handler, but 
		-- it's only done once during system initialization.
		Command.System.Watchdog.Quiet()
		-- dump(currentToon)
		
		-- Forget pre-2.4 "main bank" entries.
		if ToonInfoShard[currentToon.name]
		and ToonInfoShard[currentToon.name].slots then
			for k, v in pairs(ToonInfoShard[currentToon.name].slots) do
				if k:sub(1,4) == "sbmn" then
					ToonInfoShard[currentToon.name].slots[k]=nil
				end
			end
		end
		
		items = Inspect.Item.List()
		itemChanged(handle, items)
		currencies=Inspect.Currency.List()
		currencyChanged(handle, currencies)
		factions=Inspect.Faction.List()
		factionChanged(handle, factions)
		ToonInfoShard[currentToon.name]["character"]=currentToon
	end
	
	if ToonInfo.HackToDelayToolTip then
		if ToonInfo.HackToDelayToolTipItemName ~= nil then
			ToonInfo.showTooltipExtension(ToonInfo.HackToDelayToolTipItemName)
		else
			ToonInfo.hideTooltipExtension()		
		end
	end
end

function ToonInfo.printVersion()
	LibVersionCheck.register("ToonInfo", ToonInfo.version)
	print(L("ToonInfo Version ") .. (ToonInfo.version) .. L(" installed!"))
end

function ToonInfo.printFactions()
	local factions=Inspect.Faction.List()
	for id,noto in pairs(factions) do
		local detail=Inspect.Faction.Detail(id)
		print (id.." is "..detail.name.." have "..detail.notoriety)
	end
end

function ToonInfo.printCoin()
	local currencies=Inspect.Currency.List()
	for id,amount in pairs(currencies) do
		local detail=Inspect.Currency.Detail(id)
		print (id.." is "..detail.name.." have "..detail.stack.." max "..(detail.stackMax or "-"))
	end
end

function ToonInfo.deleteCharInfo(name)
	if name == currentToon.name then
		print (L("cannot delete information about yourself"))
		return
	end
	ToonInfoShard[name]=nil
	ToonInfo.rebuildAllWindows()
end

local function slashHandler(h, args)
	local r = {}
	local numargs = 1
	local inquote = false
	local token, tmptoken
	for token in string.gmatch(args, "[^%s]+") do
		--print(token)
		if token:sub(1, 1) == "\"" then
			tmptoken=""
			token=token:sub(2) -- handle "abc" case
			inquote=true
			--print("start qoute token="..token)
		end
		if inquote then
			--print ("in quote, last char: "..token:sub(-1))
			if token:sub(-1) == "\"" then
				inquote=false
				token=token:sub(1, -2)
				token=tmptoken .. token
				--print ("combined string is "..token)
			else
				tmptoken=tmptoken .. token .. " "
				--print ("tmp token is "..token)
			end
		end
		if not inquote then
			r[numargs] = token
			numargs=numargs+1
		end
	end
	if numargs>1 then
		if r[1] == "version" then
			ToonInfo.printVersion()
		elseif (r[1] == "delete" or r[1] == "lösche") and numargs>1 then
			if ToonInfoShard[r[2]] then
				ToonInfo.deleteCharInfo(r[2])
			else
				print (L("can't find Toon ")..r[2])
			end
		elseif (r[1] == "merge" or r[1] == "verschmelzen") and numargs > 1 then
			local newmerge=tonumber(r[2])
			if newmerge and ( newmerge==0 or newmerge==1 or newmerge==2 ) then
				ToonInfoChar.merge=tonumber(r[2])
				print (L("merge mode set to ")..ToonInfoChar.merge)
			end
		elseif r[1] == "tooltipattach" then
			ToonInfoChar.attachToTooltip=(r[2] == "true")
			print (L("attach to tooltip set to ") .. (ToonInfoChar.attachToTooltip and "true" or "false"))
		elseif r[1] == "attachposition" then
			ToonInfoChar.attachPosition = r[2]
		elseif r[1] == "coin" then
			ToonInfo.printCoin()
		elseif r[1] == "factions" then
			ToonInfo.printFactions()
		else
			print(L("Usage") .. ": /ti version | delete <char name> | merge [0|1|2] | tooltipattach [true|false] | attachposition [pos] | coin | factions")
		end
	else
		print(L("Usage") .. ": /ti version | delete <char name> | merge [0|1|2] | tooltipattach [true|false] | attachposition [pos] | coin | factions")
	end
end

local function addonLoaded(handle, addon) 
	if (addon == "ToonInfo") then
		ToonInfo.printVersion()
		ensureVariablesInited()
		ToonInfo.HackToDelayToolTip=false	-- because it might have been saved
		ToonInfo.HackToDelayToolTipSafeCounter=0
		ToonInfo.createUI()
	end
end

local function tooltipShown(handle, type, shown, buff)
	local item
	-- print ("Tooltip: type "..(type or "-nil-"))
	if (type == "item" or type == "itemtype") then
		item=Inspect.Item.Detail(shown);
		--print("Tooltip: type "..(type or "nil")..", shown "..(shown or "nil") .. ", name "..(item.name or "nil"))
		if item then
			ToonInfo.showTooltipExtension(item.name)
		end
	elseif (type==nil) then
		-- print("Tooltip: type "..(type or "-nil-")..", shown "..(shown or "nil"))
		ToonInfo.HackToDelayToolTipSafeCounter=0
		ToonInfo.hideTooltipExtension()
	else
		ToonInfo.HackToDelayToolTipItemName=nil
	end
end

local function interaction(handle, type, allowed) 
	if type == "guildbank" then
		--print ("guild bank open: " .. (allowed and "true" or "false"))
	end
end

local function guildBank(handle, vaults)
	--print "guild bank change"
	--dump(vaults)
end

Command.Event.Attach(Command.Slash.Register("ti"),      slashHandler, "Command.Slash.Register")
Command.Event.Attach(Event.Item.Slot,   		itemChanged, "ItemSlotUpdated")
Command.Event.Attach(Event.Item.Update, 		itemChanged, "ItemUpdated")
Command.Event.Attach(Event.Currency,    		currencyChanged, "CurrencyUpdated")
Command.Event.Attach(Event.Faction.Notoriety,   	factionChanged, "FactionChanged")
Command.Event.Attach(Event.System.Update.Begin, 	systemUpdate, "systemUpdate")
Command.Event.Attach(Event.Addon.Load.End, 		addonLoaded, "AddonLoaded" )
Command.Event.Attach(Event.Tooltip, 			tooltipShown, "TooltipShown")

--Command.Event.Attach(Event.Guild.Bank.Change, 		guildBank, "GuildBank")
--Command.Event.Attach(Event.Interaction, 		interaction, "Interaction")
